import logging


def configure_logging(verbose=False):
    log_level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        level=log_level,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)",
    )



import logging

import whoosh.qparser

logger = logging.getLogger(__name__)


def perform_proximity_search(ix, query_str):
    with ix.searcher() as searcher:
        parser = whoosh.qparser.MultifieldParser(["title", "content"], ix.schema)
        query = parser.parse(query_str)
        logger.debug(f"Parsed query: {query}")
        results = searcher.search(query, limit=None)
        return [dict(hit) for hit in results]



import argparse


def parse_args():
    parser = argparse.ArgumentParser(
        description="Perform proximity search on text files."
    )

    subparsers = parser.add_subparsers(dest="command", required=True)

    listindex_parser = subparsers.add_parser(
        "listindex", help="List files in index for debug"
    )
    listindex_parser.add_argument(
        "-i", "--index", type=str, default="index", help="Directory to store the index"
    )
    listindex_parser.add_argument(
        "-v", "--verbose", action="store_true", help="Enable verbose output"
    )

    index_parser = subparsers.add_parser("index", help="Index text files")
    index_parser.add_argument(
        "directories", type=str, nargs="+", help="Directories containing text files"
    )
    index_parser.add_argument(
        "-i", "--index", type=str, default="index", help="Directory to store the index"
    )
    index_parser.add_argument(
        "-e",
        "--extensions",
        type=str,
        nargs="+",
        default=["txt", "md", "org"],
        help="File extensions to index (default: txt, md, org)",
    )
    index_parser.add_argument(
        "-v", "--verbose", action="store_true", help="Enable verbose output"
    )
    index_parser.add_argument(
        "--newer",
        type=str,
        default="",
        help="Ignore files older than specified time (1h, 3.4d, 1.7y, 10s)",
    )
    index_parser.add_argument(
        "--exclude",
        type=str,
        action="append",
        default=[],
        help="Substrings to exclude from paths.",
    )
    index_parser.add_argument(
        "--ext",
        type=str,
        action="append",
        default=[],
        help="File extensions to include.",
    )

    search_parser = subparsers.add_parser("search", help="Search indexed text files")
    search_parser.add_argument("query", type=str, help="Proximity search query")
    search_parser.add_argument(
        "-i",
        "--index",
        type=str,
        default="index",
        help="Directory containing the index",
    )
    search_parser.add_argument(
        "-v", "--verbose", action="store_true", help="Enable verbose output"
    )

    config_parser = subparsers.add_parser("config", help="Show configuration")
    config_parser.add_argument(
        "-i",
        "--index",
        type=str,
        default="index",
        help="Directory containing the index",
    )
    config_parser.add_argument(
        "-v", "--verbose", action="store_true", help="Enable verbose output"
    )

    return parser.parse_args()



import pathlib
import typing

import fishhoof.find_files
import platformdirs
import whoosh.index

from . import cli, indexer, log, searcher

__project_name__ = "ourchester"


def main() -> int:
    args = cli.parse_args()
    log.configure_logging(args.verbose)

    cache_dir = _get_cache_dir()
    index_dir = cache_dir / args.index

    if args.command == "index":
        directories = [pathlib.Path(dir) for dir in args.directories]
        file_paths: typing.List[fishhoof.FileWithTimestamp] = (
            fishhoof.find_files.build_path_list(
                directories,
                excludes=args.exclude,
                extensions=args.extensions,
                newer=args.newer,
            )
        )
        file_paths = [item.file for item in file_paths]
        indexer.index_files(file_paths, index_dir)

    elif args.command == "listindex":
        indexer.get_index_files(index_dir)

    elif args.command == "search":
        try:
            index = indexer.load_index(index_dir)
            results = searcher.perform_proximity_search(index, args.query)
            _print_search_results(results)
        except whoosh.index.EmptyIndexError:
            print(f"Index does not exist in {index_dir}")
            print("Please run the 'index' command first to create the index.")
            return 1
    elif args.command == "config":
        print(f"Index directory: {index_dir}")

    return 0


def _get_cache_dir():
    user_cache_dir = platformdirs.user_cache_dir()
    ourchester_cache_dir = pathlib.Path(user_cache_dir) / "ourchester"
    ourchester_cache_dir.mkdir(exist_ok=True, parents=True)
    return ourchester_cache_dir


def _print_search_results(results):
    print(f"Found {len(results):,} documents:")
    for hit in results:
        print(f"{hit['path']}")



import logging
import pathlib
import time
import typing

import humanize
import whoosh.fields
import whoosh.index

logger = logging.getLogger(__name__)


def _create_or_open_index(index_home_path, schema):
    if whoosh.index.exists_in(index_home_path):
        return whoosh.index.open_dir(index_home_path)
    else:
        return whoosh.index.create_in(index_home_path, schema)


def _index_files(file_paths, writer):
    indexed_paths = set()
    for file_path in file_paths:
        logging.debug(f"deleting from whoosh index {file_path}")
        writer.delete_by_term("path", str(file_path))
        with open(file_path, "r") as file:
            content = file.read()
            logging.debug(f"indexing {file_path}")
            writer.add_document(
                path=str(file_path),
                title=file_path.with_suffix("").name,
                content=content,
            )
            indexed_paths.add(file_path)
    return indexed_paths


def _remove_deleted_files(ix, indexed_paths, writer):
    for doc in ix.searcher().documents():
        if pathlib.Path(doc["path"]).exists():
            continue
        if doc["path"] not in indexed_paths:
            logging.debug(f"deleting from whoosh index {doc['path']}")
            writer.delete_by_term("path", doc["path"])


def get_index_files(index_home_path):
    ix = whoosh.index.open_dir(index_home_path)

    with ix.reader() as reader:
        for stored_fields in reader.all_stored_fields():
            doc_title = stored_fields["title"]
            doc_path = stored_fields["path"]

            print(f"Title: {doc_title}")
            print(f"Path: {doc_path}")
            print("---")


def index_files(file_paths: typing.List[pathlib.Path], index_home_path):
    schema = whoosh.fields.Schema(
        title=whoosh.fields.TEXT(stored=True),
        path=whoosh.fields.ID(unique=True, stored=True),
        content=whoosh.fields.TEXT(stored=True),
    )
    start_time = time.time()
    ix = _create_or_open_index(index_home_path, schema)
    writer = ix.writer()
    indexed_paths = _index_files(file_paths, writer)
    _remove_deleted_files(ix, indexed_paths, writer)
    writer.commit()
    end_time = time.time()
    indexing_time = end_time - start_time
    t = humanize.naturaldelta(indexing_time)
    logger.info(f"Indexing completed in {t}")


def _create_or_open_index(index_dir, schema):
    if not index_dir.exists():
        index_dir.mkdir(parents=True)
        return whoosh.index.create_in(str(index_dir), schema)
    else:
        return whoosh.index.open_dir(str(index_dir))


def load_index(index_dir):
    return whoosh.index.open_dir(str(index_dir))
